 /*
  * Copyright (C), 2018-2022
  * Project: starter
  * FileName: RemoteDesignServiceKit
  * Author:   xx
  * Date:     2022/4/28 12:05
  */
 package com.fr.plugin.ihsf.kit;

 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.net.http.HttpKit;
 import com.fr.data.NetworkHelper;
 import com.fr.decision.authority.data.User;
 import com.fr.decision.config.LoginLockConfig;
 import com.fr.decision.config.PasswordStrategyConfig;
 import com.fr.decision.webservice.exception.login.UserPasswordNeedUpdateException;
 import com.fr.decision.webservice.exception.login.UserPwdErrorException;
 import com.fr.decision.webservice.utils.ControllerFactory;
 import com.fr.decision.webservice.utils.WebServiceUtils;
 import com.fr.decision.webservice.utils.controller.AuthenticController;
 import com.fr.decision.webservice.v10.login.lock.LoginLockService;
 import com.fr.decision.webservice.v10.password.strategy.PasswordStrategyService;
 import com.fr.decision.webservice.v10.remote.RemoteDesignStatusService;
 import com.fr.decision.webservice.v10.remote.RpcValidator;
 import com.fr.decision.webservice.v10.user.UserService;
 import com.fr.exception.RemoteDesignPermissionDeniedException;
 import com.fr.general.ComparatorUtils;
 import com.fr.general.GeneralUtils;
 import com.fr.general.IOUtils;
 import com.fr.intelli.record.FocusPoint;
 import com.fr.intelli.record.MetricRegistry;
 import com.fr.intelli.record.Original;
 import com.fr.json.JSONObject;
 import com.fr.locale.InterProviderFactory;
 import com.fr.plugin.ihsf.config.IhsfConfig;
 import com.fr.report.util.RemoteDesignAuthHelper;
 import com.fr.report.util.RemoteUserInfo;
 import com.fr.security.JwtUtils;
 import com.fr.security.SecurityConfig;
 import com.fr.security.encryption.SystemEncryptionManager;
 import com.fr.security.encryption.mode.EncryptionMode;
 import com.fr.security.encryption.storage.StorageEncryptors;
 import com.fr.security.encryption.transmission.TransmissionEncryptors;
 import com.fr.security.encryption.transmission.impl.SM4TransmissionEncryption;
 import com.fr.stable.StringUtils;
 import com.fr.stable.web.Device;
 import com.fr.web.service.RemoteDesignAuthorityDataService;
 import com.fr.workspace.WorkContext;

 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import java.io.IOException;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;

 /**
  * <Function Description><br>
  * <RemoteDesignServiceKit>
  *
  * @author xx
  * @since 1.0.0
  */
 public class RemoteDesignServiceKit {
     private static final String REMOTE_DESIGN_ID = "function_com.fr.remote.design";
     private static final String REMOTE_DESIGN = InterProviderFactory.getFrontProvider().getLocText("Fine-Dec_Focus_Point_Remote_Design");
     private static volatile RemoteDesignServiceKit instance = null;

     public RemoteDesignServiceKit() {
         try {
             WorkContext.registerValidator(new RpcValidator());
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
         }

     }

     public static RemoteDesignServiceKit getInstance() {
         if (instance == null) {
             synchronized (RemoteDesignServiceKit.class) {
                 if (instance == null) {
                     instance = new RemoteDesignServiceKit();
                 }
             }
         }
         return instance;
     }

     public String getRemoteToken(String username, String password, String compatibleParameters0) throws Exception {
         if (StringUtils.isEmpty(compatibleParameters0) && !ComparatorUtils.equals(StorageEncryptors.getInstance().getCurrentEncryptionMode(), EncryptionMode.RSA)) {
             throw new RemoteDesignPermissionDeniedException();
         } else {
             //SSO接口鉴权
             if (doSSOAuthenticate(username, password)) {
                 return this.generateToken(username);
             }
             boolean auth = RemoteDesignAuthHelper.doAuthenticate(username, password);
             if (auth) {
                 return this.generateToken(username);
             } else {
                 User user = UserService.getInstance().getUserByUserName(username);
                 if (user != null && RemoteDesignAuthorityDataService.getInstance().hasAuthority(user.getId())) {
                     return this.generateToken(username);
                 } else {
                     throw new RemoteDesignPermissionDeniedException();
                 }
             }
         }
     }

     /**
      * SSO接口鉴权
      *
      * @param username
      * @param password
      * @return
      */
     private boolean doSSOAuthenticate(String username, String password) {
         IhsfConfig config = IhsfConfig.getInstance();
         Map<String, String> params = new HashMap<>();
         params.put("client_id", config.getClientId());
         params.put("client_secret", config.getClientSecret());
         params.put("grant_type", config.getGrantType());
         params.put("username", username);
         params.put("password", password);
         LogKit.info("ihsf-RemoteDesignServiceKit-doAuthenticate-params:{}", params);
         try {
             String response = HttpKit.post(config.getUriBase(), params);
             LogKit.info("ihsf-RemoteDesignServiceKit-doAuthenticate-response:{}", response);
             JSONObject result = new JSONObject(response);
             return result.has("access_token");
         } catch (IOException e) {
             LogKit.error(e.getMessage(), e);
             return false;
         }
     }

     private String generateToken(String var1) throws Exception {
         String var2 = JwtUtils.createDefaultJWT(var1);
         RemoteDesignStatusService.loginStatusService().put(var2, var1, 1209600000);
         return var2;
     }

     public String saferGetRemoteToken(HttpServletRequest request) throws Exception {
         String username = NetworkHelper.getHTTPRequestParameter(request, "username");
         String password = TransmissionEncryptors.getInstance().decrypt(NetworkHelper.getHTTPRequestParameter(request, "password"));
         String ipInfo = WebServiceUtils.getIpInfoFromRequest(request);
         User user = UserService.getInstance().getUserByUserName(username);
         if (user == null) {
             throw new UserPwdErrorException();
         } else {
             String token = JwtUtils.createDefaultJWT(username);
             if (this.doSSOAuthenticate(username, password)) {
                 RemoteDesignStatusService.loginStatusService().put(token, username, 1209600000);
                 return token;
             }
             AuthenticController var6 = ControllerFactory.getInstance().getAuthenticController(user.getId());
             this.doUserAuthentication(var6, user, password, NetworkHelper.getDevice(request), ipInfo);
             if (var6.passwordChangeable(user)) {
                 this.checkUpdateInitPassword(user);
                 PasswordStrategyService.getInstance().checkPasswordStrength(password, username, token);
                 PasswordStrategyService.getInstance().checkPasswordNeedUpdate(user, token);
             }

             RemoteUserInfo var8 = RemoteDesignAuthHelper.getUserInfo(username);
             if (!var8.isRoot() && !RemoteDesignAuthorityDataService.getInstance().hasAuthority(user.getId())) {
                 throw new RemoteDesignPermissionDeniedException();
             } else {
                 RemoteDesignStatusService.loginStatusService().put(token, username, 1209600000);
                 return token;
             }
         }
     }

     private void doUserAuthentication(AuthenticController var1, User var2, String var3, Device var4, String var5) throws Exception {
         String var6 = LoginLockConfig.getInstance().getLockObject();
         String var7 = ComparatorUtils.equals(var6, "ip") ? var5 : var2.getDisplayName();
         var1.dealWithLoginLock(var2, var4, var6, var7);
         if (!var1.doAuthentication(var2, var3, var4)) {
             var1.dealWithPasswordError(var2, var4, var6, var7);
             throw new UserPwdErrorException();
         } else {
             LoginLockService.getInstance().unlockObject(var2.getId(), var6, var7);
         }
     }

     private void checkUpdateInitPassword(User var1) {
         Date var2 = var1.getPasswordChangeTime();
         if (PasswordStrategyConfig.getInstance().isUpdateInitPassword() && var2 == null) {
             throw new UserPasswordNeedUpdateException();
         }
     }

     public void recordConnection(String var1, String var2) throws Exception {
         HashMap var3 = new HashMap();
         var3.put("designerJarVersion", StringUtils.alwaysNotNull(var1));
         var3.put("serverJarVersion", GeneralUtils.readBuildNO());
         MetricRegistry.getMetric().submit(FocusPoint.newBuilder().id("function_com.fr.remote.design").text(REMOTE_DESIGN).source(Original.EMBED).username(var2).body(var3).build());
     }

     public Map<String, Object> checkInfo() {
         HashMap var1 = new HashMap();
         var1.put("frontSeed", SecurityConfig.getInstance().getFrontSeed());
         var1.put("frontSM4Key", SM4TransmissionEncryption.getInstance().getTransmissionKey());
         var1.put("transmissionEncryption", SystemEncryptionManager.getInstance().getTransmissionEncryption().getType());
         return var1;
     }

     public void onMessage(HttpServletRequest var1, HttpServletResponse var2) throws Exception {
         byte[] var3 = IOUtils.inputStream2Bytes(var1.getInputStream());
         var2.getOutputStream().write(WorkContext.handleMessage(var3));
     }
 }