 /*
  * Copyright (C), 2018-2021
  * Project: starter
  * FileName: RemoteFilter
  * Author:   xx
  * Date:     2021/3/30 22:09
  */
 package com.fr.plugin.ihsf.request;

 import com.fanruan.api.i18n.I18nKit;
 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.util.StringKit;
 import com.fr.data.NetworkHelper;
 import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
 import com.fr.decision.webservice.Response;
 import com.fr.intelli.record.Focus;
 import com.fr.intelli.record.Original;
 import com.fr.intelligence.IntelligenceException;
 import com.fr.intelligence.IntelligenceRuntimeException;
 import com.fr.json.JSONObject;
 import com.fr.plugin.context.PluginContexts;
 import com.fr.plugin.ihsf.kit.RemoteDesignServiceKit;
 import com.fr.record.analyzer.EnableMetrics;
 import com.fr.security.SecurityToolbox;
 import com.fr.stable.fun.Authorize;
 import com.fr.web.utils.WebUtils;

 import javax.servlet.FilterChain;
 import javax.servlet.FilterConfig;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;

 import static com.fr.plugin.ihsf.LocaleFinder.PLUGIN_ID;

 /**
  * <Function Description><br>
  * <RemoteFilter>
  *
  * @author xx
  * @since 1.0.0
  */
 @Authorize(callSignKey = PLUGIN_ID)
 @EnableMetrics
 public class RemoteFilter extends AbstractGlobalRequestFilterProvider {

     /**
      * 过滤器名称
      *
      * @return
      */
     @Override
     public String filterName() {
         return "ihsfFilter";
     }

     /**
      * 过滤规则
      *
      * @return
      */
     @Override
     @Focus(id = PLUGIN_ID, text = "Plugin-ihsf", source = Original.PLUGIN)
     public String[] urlPatterns() {
         if (PluginContexts.currentContext() == null || !PluginContexts.currentContext().isAvailable()) {
             LogKit.error(I18nKit.getLocText("Plugin-ihsf_Licence_Expired"));
             return new String[]{};
         }
         return new String[]{"/decision/remote/design/token", "/decision/remote/design/verify"};
     }

     /**
      * 过滤器初始化
      *
      * @param filterConfig
      */
     @Override
     public void init(FilterConfig filterConfig) {
         super.init(filterConfig);
     }

     /**
      * 过滤器处理
      *
      * @param request
      * @param response
      * @param filterChain
      */
     @Override
     public void doFilter(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) {
         try {
             String pathInfo = (request.getPathInfo() != null) ? request.getPathInfo() : StringKit.EMPTY;
             if (StringKit.equals(pathInfo, "/remote/design/token")
                     && StringKit.equalsIgnoreCase(request.getMethod(), "get")) {
                 getRemoteToken(request, response);
             } else if (StringKit.equals(pathInfo, "/remote/design/token")
                     && StringKit.equalsIgnoreCase(request.getMethod(), "post")) {
                 safeGetRemoteToken(request, response);
             } else if (StringKit.equals(pathInfo, "/remote/design/verify")
                     && StringKit.equalsIgnoreCase(request.getMethod(), "post")) {
                 saferGetRemoteToken(request, response);
             } else {
                 filterChain.doFilter(request, response);
             }
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
         }
     }

     private void getRemoteToken(HttpServletRequest request, HttpServletResponse response) throws Exception {
         LogKit.info("ihsf-RemoteFilter-getRemoteToken");
         String username = NetworkHelper.getHTTPRequestParameter(request, "username");
         String password = NetworkHelper.getHTTPRequestParameter(request, "password");
         String compatibleParameters0 = NetworkHelper.getHTTPRequestParameter(request, "compatibleParameters0");
         Response result;
         try {
             result = Response.ok(RemoteDesignServiceKit.getInstance().getRemoteToken(username, password, compatibleParameters0));
             WebUtils.printAsString(response, JSONObject.mapFrom(result).encode());
         } catch (IntelligenceRuntimeException | IntelligenceException exception) {
             result = Response.error((exception).errorCode(), exception.getMessage());
             WebUtils.printAsString(response, JSONObject.mapFrom(result).encode());
         }
     }

     private void safeGetRemoteToken(HttpServletRequest request, HttpServletResponse response) throws Exception {
         LogKit.info("ihsf-RemoteFilter-safeGetRemoteToken");
         String username = NetworkHelper.getHTTPRequestParameter(request, "username");
         String password = SecurityToolbox.defaultDecrypt(NetworkHelper.getHTTPRequestParameter(request, "password"));
         String compatibleParameters0 = NetworkHelper.getHTTPRequestParameter(request, "compatibleParameters0");
         Response result;
         try {
             result = Response.ok(RemoteDesignServiceKit.getInstance().getRemoteToken(username, password, compatibleParameters0));
             WebUtils.printAsString(response, JSONObject.mapFrom(result).encode());
         } catch (IntelligenceRuntimeException | IntelligenceException exception) {
             result = Response.error((exception).errorCode(), exception.getMessage());
             WebUtils.printAsString(response, JSONObject.mapFrom(result).encode());
         }
     }

     private void saferGetRemoteToken(HttpServletRequest request, HttpServletResponse response) throws Exception {
         LogKit.info("ihsf-RemoteFilter-saferGetRemoteToken");
         Response result;
         try {
             result = Response.ok(RemoteDesignServiceKit.getInstance().saferGetRemoteToken(request));
             WebUtils.printAsString(response, JSONObject.mapFrom(result).encode());
         } catch (IntelligenceRuntimeException | IntelligenceException exception) {
             result = Response.error((exception).errorCode(), exception.getMessage());
             WebUtils.printAsString(response, JSONObject.mapFrom(result).encode());
         }
     }
 }